/// This code was generated by
/// \ / _    _  _|   _  _
///  | (_)\/(_)(_|\/| |(/_  v1.0.0
///       /       /

using System;
using System.Collections.Generic;
using System.Linq;
using Twilio.Base;
using Twilio.Converters;

namespace Twilio.Rest.Conversations.V1
{

    /// <summary>
    /// Create a new user role in your account's default service
    /// </summary>
    public class CreateRoleOptions : IOptions<RoleResource>
    {
        /// <summary>
        /// A string to describe the new resource
        /// </summary>
        public string FriendlyName { get; }
        /// <summary>
        /// The type of role
        /// </summary>
        public RoleResource.RoleTypeEnum Type { get; }
        /// <summary>
        /// A permission the role should have
        /// </summary>
        public List<string> Permission { get; }

        /// <summary>
        /// Construct a new CreateRoleOptions
        /// </summary>
        /// <param name="friendlyName"> A string to describe the new resource </param>
        /// <param name="type"> The type of role </param>
        /// <param name="permission"> A permission the role should have </param>
        public CreateRoleOptions(string friendlyName, RoleResource.RoleTypeEnum type, List<string> permission)
        {
            FriendlyName = friendlyName;
            Type = type;
            Permission = permission;
        }

        /// <summary>
        /// Generate the necessary parameters
        /// </summary>
        public List<KeyValuePair<string, string>> GetParams()
        {
            var p = new List<KeyValuePair<string, string>>();
            if (FriendlyName != null)
            {
                p.Add(new KeyValuePair<string, string>("FriendlyName", FriendlyName));
            }

            if (Type != null)
            {
                p.Add(new KeyValuePair<string, string>("Type", Type.ToString()));
            }

            if (Permission != null)
            {
                p.AddRange(Permission.Select(prop => new KeyValuePair<string, string>("Permission", prop)));
            }

            return p;
        }
    }

    /// <summary>
    /// Update an existing user role in your account's default service
    /// </summary>
    public class UpdateRoleOptions : IOptions<RoleResource>
    {
        /// <summary>
        /// The SID of the Role resource to update
        /// </summary>
        public string PathSid { get; }
        /// <summary>
        /// A permission the role should have
        /// </summary>
        public List<string> Permission { get; }

        /// <summary>
        /// Construct a new UpdateRoleOptions
        /// </summary>
        /// <param name="pathSid"> The SID of the Role resource to update </param>
        /// <param name="permission"> A permission the role should have </param>
        public UpdateRoleOptions(string pathSid, List<string> permission)
        {
            PathSid = pathSid;
            Permission = permission;
        }

        /// <summary>
        /// Generate the necessary parameters
        /// </summary>
        public List<KeyValuePair<string, string>> GetParams()
        {
            var p = new List<KeyValuePair<string, string>>();
            if (Permission != null)
            {
                p.AddRange(Permission.Select(prop => new KeyValuePair<string, string>("Permission", prop)));
            }

            return p;
        }
    }

    /// <summary>
    /// Remove a user role from your account's default service
    /// </summary>
    public class DeleteRoleOptions : IOptions<RoleResource>
    {
        /// <summary>
        /// The SID of the Role resource to delete
        /// </summary>
        public string PathSid { get; }

        /// <summary>
        /// Construct a new DeleteRoleOptions
        /// </summary>
        /// <param name="pathSid"> The SID of the Role resource to delete </param>
        public DeleteRoleOptions(string pathSid)
        {
            PathSid = pathSid;
        }

        /// <summary>
        /// Generate the necessary parameters
        /// </summary>
        public List<KeyValuePair<string, string>> GetParams()
        {
            var p = new List<KeyValuePair<string, string>>();
            return p;
        }
    }

    /// <summary>
    /// Fetch a user role from your account's default service
    /// </summary>
    public class FetchRoleOptions : IOptions<RoleResource>
    {
        /// <summary>
        /// The SID of the Role resource to fetch
        /// </summary>
        public string PathSid { get; }

        /// <summary>
        /// Construct a new FetchRoleOptions
        /// </summary>
        /// <param name="pathSid"> The SID of the Role resource to fetch </param>
        public FetchRoleOptions(string pathSid)
        {
            PathSid = pathSid;
        }

        /// <summary>
        /// Generate the necessary parameters
        /// </summary>
        public List<KeyValuePair<string, string>> GetParams()
        {
            var p = new List<KeyValuePair<string, string>>();
            return p;
        }
    }

    /// <summary>
    /// Retrieve a list of all user roles in your account's default service
    /// </summary>
    public class ReadRoleOptions : ReadOptions<RoleResource>
    {
        /// <summary>
        /// Generate the necessary parameters
        /// </summary>
        public override List<KeyValuePair<string, string>> GetParams()
        {
            var p = new List<KeyValuePair<string, string>>();
            if (PageSize != null)
            {
                p.Add(new KeyValuePair<string, string>("PageSize", PageSize.ToString()));
            }

            return p;
        }
    }

}